package br.usp.ime.prolog.editor;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.File;
import java.util.Vector;

import br.usp.ime.prolog.PrologPlugin;
import br.usp.ime.prolog.preferences.IPrologPreferenceFields;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;

/**
 * 
 * @author Juliana
 *
 * The 'Scanner' for Prolog source code.
 * 
 */
public class PrologRuleScanner extends RuleBasedScanner {
	private static String predFilename = "editor" + File.separator + "builtinList.txt";	
	private static String operFilename = "editor" + File.separator + "operatorsList.txt";
	
	private static Vector  fgPredicates = new Vector();
	private static Vector  fgOperators  = new Vector();

	public PrologRuleScanner(TokenManager tokenManager) {
		
		IToken commentToken   = 
			tokenManager.getToken(IPrologPreferenceFields.PREF_COMMENT_COLOR);
		IToken predicateToken = 
			tokenManager.getToken(IPrologPreferenceFields.PREF_BUILTIN_COLOR);
		IToken varToken	      = 
			tokenManager.getToken(IPrologPreferenceFields.PREF_VAR_COLOR);
		IToken operToken	  = 
			tokenManager.getToken(IPrologPreferenceFields.PREF_OPER_COLOR);
		IToken stringToken    = 
			tokenManager.getToken(IPrologPreferenceFields.PREF_STRING_COLOR);
		IToken other		  = 
			tokenManager.getToken(IPrologPreferenceFields.PREF_DEFAULT_COLOR);

		Vector rules = new Vector();
		
		// Rule for comments. 		
		rules.add(new EndOfLineRule("%", commentToken));
		
		// Rule for white spaces.
		rules.add(new WhitespaceRule(new PrologWhitespaceDetector()));

		// Rule for constants between single quotes.		
		rules.add(new MultiLineRule("'", "'", other, '\\'));

		// Rule for strings.
		rules.add(new MultiLineRule("\"", "\"", stringToken, '\\'));
		
		// Rule for variables.
		WordRule varRule = new WordRule(new PrologVarDetector(), varToken);
		rules.add(varRule);
					
		// Rules for operators.		 
		WordRule operatorRule= new WordRule(new PrologOperatorDetector(), other);
		// The Prolog operators are listed in 'operFilename'.
		// We have to read them to the vector 'fgOperators'.
		getKeywords(fgOperators,operFilename);
		for (int i= 0; i < fgOperators.size(); i++)		
			operatorRule.addWord((String) fgOperators.get(i), operToken);
		rules.add(operatorRule);		

		// Rule for built-in predicates. 			 
		WordRule predicateRule= new WordRule(new PrologWordDetector(), other);
		// The Prolog built-in predicates are listed in 'predFilename'.
		// We have to read them to the vector 'fgPredicates'.
		getKeywords(fgPredicates,predFilename);
		for (int i= 0; i < fgPredicates.size(); i++) {
			predicateRule.addWord((String) fgPredicates.get(i), predicateToken);
		}	
		rules.add(predicateRule); 				
		
		
		IRule[] result = new IRule[rules.size()];
		rules.toArray(result);	
		setRules(result);			 	
	}
	
	
	/**
	 * Populates the vector 'v' with the keywords inside
	 * 'filename'.
	 * 'filename' must be a relative path from the plugin
	 *  directory instalation.
	 * 
	 * @param v 'Vector' to be populated with the constants
	 *        in 'filename'.
	 * 
	 * @param filename Path and filename where we can find
	 *        the constants to be recognized.
	 * 
	 */	
	private void getKeywords(Vector v, String filename) {
		String keyword;

		try {
			// Criar caminho para 'filename', relativo
			// ao diretorio de instalacao do plugin.
			Path path = new Path(filename);
			Plugin plugin = PrologPlugin.getDefault();
		
			InputStream inFile = null;
			try {
				// Obter um InputStream para 'filename'.			
				inFile = plugin.openStream(path);							
			}
			catch(IOException ioe) {
				throw (new IOException("Error opening file: " + filename + "."));
			}
			
			// Criar leitura bufferizada.			
			BufferedReader in = new BufferedReader (new InputStreamReader(inFile));						
			
			// Ler palavras chaves e popular vetor parametro.
			while ((keyword = in.readLine())!= null) {				
				v.add(keyword);			
			}						
			in.close();
			inFile.close();
		}
		catch (IOException e) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Problems doing lexical analysis: " +
			   e.getMessage(), e);
			PrologPlugin.log(status);	   
			
		}
		catch (Exception e2) {
			// Log the error to the Eclipse log.
			IStatus status = new Status(IStatus.ERROR,
			   PrologPlugin.getDefault().getDescriptor().getUniqueIdentifier(),0,
			   "Problems doing lexical analysis: " +
			   e2.getMessage(), e2);
			PrologPlugin.log(status);	   
		}				 
	}	

}
